package com.github.huangp.learning.datastructure;

/**
 * @author Patrick Huang
 *         <a href="mailto:pahuang@redhat.com">pahuang@redhat.com</a>
 */
public class BinarySearchTree<K extends Comparable<K>, V> {
    private Node<K, V> root;

    public BinarySearchTree(
            Node<K, V> root) {
        this.root = root;
    }

    public Node<K, V> put(K key, V value) {
        return put(root, key, value);
    }

    private Node<K, V> put(Node<K, V> node, K key, V value) {
        if (node == null) {
            return new Node<K, V>(key, value, null, null, 1);
        }
        if (key.compareTo(node.key) < 0) {
            node.left = put(node.left, key, value);
        } else if (key.compareTo(node.key) > 0) {
            node.right = put(node.right, key, value);
        } else {
            node.value = value;
            return node;
        }
        node.numOfNodes = size(node.left) + size(node.right) + 1;
        return node;
    }

    private int size(Node<K, V> node) {
        return node == null ? 0 : node.numOfNodes;
    }

    public boolean contains(K key) {
        int compareResult = root.key.compareTo(key);
        if (compareResult > 0) {
            return contains(root.left, key);
        } else if (compareResult < 0) {
            return contains(root.right, key);
        } else {
            return true;
        }
    }

    private boolean contains(Node<K, V> node, K key) {
        if (node == null) {
            return false;
        }
        int compareResult = node.key.compareTo(key);
        if (compareResult > 0) {
            return contains(node.left, key);
        } else if (compareResult < 0) {
            return contains(node.right, key);
        } else {
            return true;
        }
    }


    private class Node<K, V> {
        private K key;
        private V value;
        private Node<K, V> left;
        private Node<K, V> right;
        private int numOfNodes; // in subtree

        private Node(K key, V value,
                Node<K, V> left,
                Node<K, V> right, int numOfNodes) {
            this.key = key;
            this.value = value;
            this.left = left;
            this.right = right;
            this.numOfNodes = numOfNodes;
        }
    }
}
