package com.github.huangp.learning.ga.travellingsalemanproblem;

import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Random;

import com.google.common.base.Functions;
import com.google.common.base.Joiner;
import com.google.common.collect.ImmutableList;
import com.google.common.collect.Iterables;
import com.google.common.collect.Lists;

/**
 * @author Patrick Huang
 *         <a href="mailto:pahuang@redhat.com">pahuang@redhat.com</a>
 */
public class CandidateTour implements Individual<CandidateTour> {
    private List<City> tour = Lists.newArrayList();
    // cache
    private double fitness = 0;
    private double distance = 0;

    private CandidateTour() {
    }

    public static CandidateTour newTour() {
        return new CandidateTour().generateIndividual();
    }

    public static CandidateTour emptyTour() {
        CandidateTour candidateTour = new CandidateTour();
        for (int i = 0; i < DestinationCities.manager().numOfCities(); i++) {
            candidateTour.tour.add(null);
        }
        return candidateTour;
    }

    @Override
    public CandidateTour generateIndividual() {
        tour = Lists.newArrayList(Iterables.transform(DestinationCities.manager().getCities(),
                Functions.<City>identity()));
        Collections.shuffle(tour, new Random(new Date().getTime()));
        return this;
    }

    public City getCity(int index) {
        return tour.get(index);
    }

    public void setCity(int index, City city) {
        tour.set(index, city);
        // we have changed the tour therefore need to reset cache
        fitness = 0;
        distance = 0;
    }

    @Override
    public double fitness() {
        if (fitness == 0) {
            fitness = 1 / totalTravellingDistance();
        }
        return fitness;
    }

    public double totalTravellingDistance() {
        if (distance == 0) {
            int numOfCities = tour.size();
            for (int i = 0; i < numOfCities; i++) {
                City from = tour.get(i);
                City to;
                if (i == numOfCities - 1) {
                    to = tour.get(0);
                } else {
                    to = tour.get(i + 1);
                }
                distance += from.distance(to);
            }
        }
        return distance;
    }

    public boolean containsCity(City city) {
        return tour.contains(city);
    }

    @Override
    public String toString() {
        return Joiner.on(" -> ").join(tour);
    }

    public List<City> getCities() {
        return ImmutableList.copyOf(tour);
    }
}
