package com.github.huangp.learning.datastructure;

import org.assertj.core.api.Assertions;
import org.junit.Before;
import org.junit.Test;

/**
 * @author Patrick Huang
 *         <a href="mailto:pahuang@redhat.com">pahuang@redhat.com</a>
 */
public class HashMapTest {
    private HashMap<String, Integer> map;

    @Before
    public void setUp() {
        map = new HashMap<String, Integer>();
    }
    @Test
    public void testPut() throws Exception {
        map.put("a", 1);
        map.put("b", 2);
        Assertions.assertThat(map.size()).isEqualTo(2);
        Assertions.assertThat(map.keys()).contains("a", "b");
        System.out.println(map);
    }

    @Test
    public void testPutWithSameIndex() {
        System.out.println("f".hashCode() % 20);
        System.out.println("z".hashCode() % 20);
        map.put("f", 1);
        map.put("z", 2);
        Assertions.assertThat(map.size()).isEqualTo(2);
        Assertions.assertThat(map.keys()).contains("f", "z");
        System.out.println(map);
        Assertions.assertThat(map.get("f")).isEqualTo(1);
        Assertions.assertThat(map.get("z")).isEqualTo(2);
    }

    @Test
    public void putWithSameKey() {
        map.put("a", 1);
        Assertions.assertThat(map.put("a", 2)).isEqualTo(1);
        Assertions.assertThat(map.keys()).contains("a");
        Assertions.assertThat(map.size()).isEqualTo(1);
    }

    @Test
    public void testContains() throws Exception {
        map.put("a", 1);
        Assertions.assertThat(map.contains("a")).isTrue();
        Assertions.assertThat(map.contains("b")).isFalse();
    }

    @Test
    public void testRemove() throws Exception {
        map.put("a", 1);
        map.put("b", 2);
        Assertions.assertThat(map.remove("a")).isTrue();
        Assertions.assertThat(map.remove("z")).isFalse();
        Assertions.assertThat(map.size()).isEqualTo(1);
        Assertions.assertThat(map.keys()).containsExactly("b");

    }

    @Test
    public void testSize() throws Exception {
        for (int i = 0; i < 100; i++) {
            map.put("" + i, i + 20);
        }
        Assertions.assertThat(map.size()).isEqualTo(100);
        System.out.println(map);
    }

    @Test
    public void testGet() throws Exception {
        Assertions.assertThat(map.put("a", 1)).isNull();
        Assertions.assertThat(map.get("a")).isEqualTo(1);
        Assertions.assertThat(map.get("b")).isNull();
    }
}
